--[[
DHUD3
Copyright (c) 2006 by Markus Inger, 2006 by Caeryn Dryad, 2007-2009 by Horacio Hoyos

This file is part of DHUD3.

    DHUD3 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DHUD3 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DHUD3.  If not, see <http://www.gnu.org/licenses/>.
]]
--Addon Mix
DHUD3 = LibStub("AceAddon-3.0"):NewAddon("DHUD3", "AceConsole-3.0", "LibDebugLog-1.0");

--ACE Version
local VERSION = tonumber(("$Rev: 0$"):match("%d+"));
DHUD3.revision = "r" .. VERSION;
DHUD3.versionstring = "3.0 |cffff8888r%d|r";
DHUD3.version = DHUD3.versionstring:format(VERSION);
DHUD3.date = ("$Date: 2008-12-16 19:58:31 -0500 (Mar, 16 Dic 2008) $"):match("%d%d%d%d%-%d%d%-%d%d");
local L = LibStub("AceLocale-3.0"):GetLocale("DHUD3");

local unpack = unpack;
local pairs = pairs;
local type = type;
local _G = _G;
local string_format = string.format;
local string_len = string.len;
local UnitHealthMax = UnitHealthMax;
local UnitHealth = UnitHealth;
local UnitInParty = UnitInParty;
local UnitManaMax = UnitManaMax;
local UnitMana = UnitMana;
local UnitPowerType = UnitPowerType;
local UnitIsPlayer = UnitIsPlayer;
local UnitPlayerControlled = UnitPlayerControlled;
local UnitCanAttack = UnitCanAttack;
local UnitIsUnit = UnitIsUnit;

local isSetup = false;
local currentAlphaValue = 10;

local mainlayout = {
    mainFrame = {
        "CENTER",
        "UIParent",
        "CENTER",
        0,
        0,
        512,
        256
    },
    -- Frames: 2 main frames to contain the bars and text for each of the sides
    frames = {
        leftFrame = {
            "LEFT",
            "mainFrame",
            "LEFT",
            0,
            0,
            128,
            256,
            0,
            1,
            0,
            1,
            "Interface\\AddOns\\DHUD3\\textures\\bg_21pp",
        },
        rightFrame = {
            "RIGHT", 
            "mainFrame",
            "RIGHT",
            0,
            0,
            128,
            256,
            1,
            0,
            0,
            1,
            "Interface\\AddOns\\DHUD3\\textures\\bg_21",
        }
    },
}

local optGetter, optSetter;
do
	function optGetter(info)
		local key = info[#info];
		return DHUD3.db.profile[key];
	end
	
	function optSetter(info, value)
		local key = info[#info];
		DHUD3.db.profile[key] = value;
		DHUD3:Refresh();
	end
end

--Local Defaults
local defaults = {
	profile = {
        blizCasting = false,
		blizPet = false,
		blizPlayer = false,
		blizTarget = false,
		deadAlpha = 0,
        combatAlpha = 1,
		castAlpha = 0.8,
        selectAlpha = 0.6,
        regenAlpha = 0.4,
        oocAlpha = 0.1,
		useBorders = true,
		showEmpty = true,
		scale = 1,
        modules = {
			['*'] = true,
		},
    }
};

--Local Options
local options, moduleOptions = nil, {}
local function getOptions()
	--if not options then
        --Options table
    	options = {
    		type = "group",
            name = "DHUD3",
            order = 1,
    		args = {
                layout = {
                    type = "group",
                    name = L["General Options"],
    				order = 1,
                    get = optGetter,
                    set = optSetter,
                    args = {
                        scale = {
                            order = 1,
                            type = "range",
                            name = L["Set DHUD3's scale"],
                            min = 0.2,
                            max = 2,
                            step = 0.1,
                        },
                        gral = {
                            type = 'group',
                            name = "General",
                            inline = true,
                            order = 2,
                            args = {
                                desc = {
            						order = 2,
                                    type = "description",
            						name = L["Configure bar's border functionalily"],
                                },
                                useBorders = {
                                    order = 3,
                                    type = "toggle",
                                    name = L["Show Bar Borders"],
                                    desc = L["Show/hide borders"],
                                },
                                showEmpty = {
                                    order = 4,
                                    type = "toggle",
                                    name = L["Show Bars when empty"],
                                    desc = L["Show/hide empty (0 value) bars"],
                                    disabled = function()
                                            return not db.useBorders
                                        end,
                                },
                            },
                        },
                        alphas = {
                            order = 3,
                            type = "group",
                            name = L["Alphas"],
                            inline = true,
                            args = {
                                desc = {
            						order = 1,
                                    type = "description",
            						name = L["Set the HUD's situation alpha. The order shown is the presedence order, i.e. Combat alpha will override Selection alpha."],
                                },
                                deadAlpha = {
                                    order = 2,
                                    type = "range",
                                    name = L["Death"],
                                    min = 0,
                                    max = 1,
                                    step = 0.05,
                                },
                                combatAlpha = {
                                    order = 3,
                                    type = "range",
                                    name = L["Combat"],
                                    min = 0,
                                    max = 1,
                                    step = 0.05,
                                },
                                castAlpha = {
                                    order = 4,
                                    type = "range",
                                    name = L["Casting"],
                                    min = 0,
                                    max = 1,
                                    step = 0.05,
                                },
                                selectAlpha = {
                                    order = 5,
                                    type = "range",
                                    name = L["Selection"],
                                    min = 0,
                                    max = 1,
                                    step = 0.05,
                                },
                                regenAlpha = {
                                    order = 6,
                                    type = "range",
                                    name = L["Regeneration"],
                                    min = 0,
                                    max = 1,
                                    step = 0.05,
                                },
                                oocAlpha = {
                                    order = 7,
                                    type = "range",
                                    name = L["Off Combat"],
                                    min = 0,
                                    max = 1,
                                    step = 0.05,
                                },
                            },
                        },
                        show = {
                            type = 'group',
    						order = 4,
                            name = L["Bliz Frames"],
                            inline = true,
                            args = {
                                desc = {
            						order = 1,
                                    type = "description",
            						name = L["Show/Hide bliz frames"],
                                },
                                blizCasting = {
                                    type ='toggle',
                                    name = L["Casting Bar"],
                                    desc = L["Show Blizzard casting bar"],
                                },
                                blizPet = {
                                    type ='toggle',
                                    name = L["Pet Frame"],
                                    desc = L["Show Blizzard's pet frame"],
                                },
                                blizPlayer = {
                                    type ='toggle',
                                    name = L["Player Frame"],
                                    desc = L["Show Blizzard's player frame"],
                                },
                                blizTarget = {
                                    type ='toggle',
                                    name = L["Target Frame"],
                                    desc = L["Show Blizzard's target frame"],
                                },
                            },
                        },
                    },
                }
            }
        }
        for k,v in pairs(moduleOptions) do
			options.args[k] = (type(v) == "function") and v() or v;
		end
    --end
    
    return options;
end
--FIN

--FIN
    
-- Local Functions
local myAddons;
do
    function myAddons()
        if (myAddOnsFrame_Register) then
            local DHUD3_mya = {
                ["name"]         = "DHUD3",
                ["version"]      = versionstring,
                ["author"]       = "Arcanefoam (Horacio Hoyos) -- Drathl/Silberklinge",
                ["category"]     = MYADDONS_CATEGORY_COMBAT,
                ["email"]        = "arcanefoam@gmail.com",
                ["website"]      = "http://www.wowace.com/projects/DHUD3/",
            };
            myAddOnsFrame_Register(DHUD3_mya);
        end
    end

end

--[[
        getHealth: Get the health value for a unit. 
        arguments:
            unit: unit to query value
        returns:
            maximum, current and percent values. need: bollean idicating if the unit's health is below 100% --]]
function DHUD3:getUnitHealth(unit)
    local maxVal, curVal, curFrac, need, found;
    maxVal = UnitHealthMax(unit);
    curVal = UnitHealth(unit);
    curFrac = curVal / maxVal;
    if curFrac ~= 1 then 
        need = true;
    end
    return maxVal, curFrac, need;
end

--[[
        getPower: Get the health value for a unit. 
        arguments:
            unit: unit to query value
            powerType: type of power for that unit
        returns:
            maximum, current and percent values. nedd: bollean idicating if the unit lackcs power (has power if power = rage) --]]
function DHUD3:getUnitPower(unit)
    local maxVal, curVal, curFrac, need, powerType;
    powerType = UnitPowerType(unit);
    if ( powerType ) then
        curVal = UnitPower(unit, powerType);
		maxVal = UnitPowerMax(unit, powerType);
        if ( maxVal ~= 0 ) then
            curFrac = curVal / maxVal;
            if ( powerType == 1 ) then
                if ( curFrac ~= 0 ) then 
                    need = true;
                end
            else
                if ( curFrac ~= 1 ) then 
                    need = true;
                end
            end
        else
            curFrac = 0;
            need = false;
        end
        return maxVal, curFrac, need;
    else
        return false;
    end
end

 --[[
            updateAlpha: Set the HUD's alpha acording to the state of the player
            arguments:
            returns:
                --]]
function DHUD3:updateAlpha(newAlpha)
    if newAlpha > currentAlphaValue then
        local alpha;
        currentAlphaValue = newAlpha;
        local p = self.db.profile;
        if ( newAlpha == 5 ) then
            alpha = "deadAlpha";
        elseif ( newAlpha == 4 ) then
            alpha = "combatAlpha";
        elseif ( newAlpha == 3 ) then
    		alpha = "castAlpha";
        elseif ( newAlpha == 2 ) then
            alpha = "selectAlpha";
        elseif ( newAlpha == 1 ) then
            alpha = "regenAlpha";
    	elseif ( newAlpha == 0 ) then
            alpha = "oocAlpha";
        end
        DHUD3_leftFrame:SetAlpha(p[alpha]);
        DHUD3_rightFrame:SetAlpha(p[alpha]);
    end
end

--[[
            rgb2hex: Transform a RGB color to a HEX color
            arguments:
                r, g, b: RGB values
            returns:
                Hex value for the RGB   --]]
function DHUD3:rgb2hex(r, g, b)
    local hexr, hexg, hexb;
    hexr = string_format("%x", r * 255);
    hexr = string_len(hexr) == 1 and "0"..hexr or hexr;
    hexg = string_format("%x", g * 255);
    hexg = string_len(hexg) == 1 and "0"..hexg or hexg;
    hexb = string_format("%x", b * 255);
    hexb = string_len(hexb) == 1 and "0"..hexb or hexb;
    return hexr..hexg..hexb;
end

--[[
        colorize: Get the bar color according to the mana or power type and percent full. Used for bars and texts.
        arguments:
            who: Unit's property being colorized
            propoerty: Property being colorized
            percent: Current percent value for that property
        returns:
            r, g, b and hex values for the calculated color.    --]]
function DHUD3:colorize(colorTable, percent)

	local r, g, b, diff, r0, g0, b0, r1, g1, b1, r2, g2, b2;
    local threshold1, threshold2  = 0.6, 0.2;
    if ( colorTable ) then
        r0, g0, b0 = unpack(colorTable.full);
        r1, g1, b1 = unpack(colorTable.med);
        r2, g2, b2 = unpack(colorTable.low);
    end
    if ( percent <= threshold2 ) then
        r, g, b = r2, g2, b2;
    elseif ( percent <= threshold1) then
        diff = 1 - (percent - threshold2) / (threshold1 - threshold2);
        r = r1 - (r1 - r2) * diff;        
        g = g1 - (g1 - g2) * diff;        
        b = b1 - (b1 - b2) * diff;    
    elseif ( percent < 1) then        
        diff = 1 - (percent - threshold1) / (1 - threshold1);        
        r = r0 - (r0 - r1) * diff;        
        g = g0 - (g0 - g1) * diff;        
        b = b0 - (b0 - b1) * diff;    
    else
        r, g, b = r0, g0, b0;
    end
	----self:Debug("colorize", r, g, b)
    if (r < 0) then r = 0; end    
    if (r > 1) then r = 1; end    
    if (g < 0) then g = 0; end    
    if (g > 1) then g = 1; end    
    if (b < 0) then b = 0; end    
    if (b > 1) then b = 1; end
    hex = self:rgb2hex(r, g, b);
    return r, g, b, hex;
end

 --[[
        unitIsNPC: Check if the unit is a NPC
        arguments:
            unit: unit to check
        returns:
            true if the unit is a NPC --]]
function DHUD3:unitIsNPC(unit)
    return not (UnitIsPlayer(unit) or UnitPlayerControlled(unit) or UnitCanAttack("player", unit));
end

--[[
        unitIsPet: Check if the unit is a pet
        arguments:
            unit: unit to check
        returns:
            true if the unit is a pet --]]
function DHUD3:unitIsPet(unit)
    return not self:unitIsNPC(unit) and not UnitIsPlayer(unit) and UnitPlayerControlled(unit);
end   

--[[ 
        updateBar: Update a bars value, changing its height and color acording to the maximun and current value
        arguments:
            owner: unit tha bar displays info about
            bar:    bar to set value
            valueType: identify if it is health or power
]]--
function DHUD3:updateBar(bar, maxVal, fracVal, color)
    --DHUD3:Debug("updateBar", bar, maxVal, fracVal, color)
    local bgt = _G["DHUD3_"..bar.."_Texture"];
    if ( maxVal == 0 ) then
        _G["DHUD3_"..bar.."_Text_Text"]:Hide();
    else
        _G["DHUD3_"..bar.."_Text_Text"]:Show();
        _G["DHUD3_"..bar.."_Text_Text"]:SetTextColor(color.r, color.g, color.b)
    end
    
    -- Hide when Bar empty ?
    if ( fracVal == 0 ) then
        bgt:Hide()
        if ( not ( db.showEmpty and db.useBorders ) ) then
            _G["DHUD3_"..bar.."_Border"]:Hide();
        else
            _G["DHUD3_"..bar.."_Border"]:Show();
        end
    else
        local point, parent, relative, position, texture, border, gap
        if DHUD3.layout.playerBars[bar] then
            point, parent, relative, position, texture, border, gap = unpack(DHUD3.layout.playerBars[bar]);
        end
        local tex_gap_top, tex_gap_bottom = unpack(DHUD3.layout[gap]);
        local x0, x1, y0, y1 = unpack(DHUD3.layout[position]);
        local tex_gap_top_p    = tex_gap_top / 256
        local tex_gap_bottom_p = tex_gap_bottom / 256
        local h = (256 - tex_gap_top - tex_gap_bottom) * fracVal
        local top    = 1 - ( fracVal - (tex_gap_top_p) )
        local bottom = 1 - tex_gap_bottom_p
        top = top  - ( (tex_gap_top_p + tex_gap_bottom_p) * (1 - fracVal) )
        if ( top > 1 ) then top = 1; end
        bgt:SetHeight(h)
        bgt:SetTexCoord(x0, x1, top, bottom )
        bgt:SetPoint(point, _G["DHUD3_"..bar], relative, 0, tex_gap_bottom)
        bgt:SetVertexColor(color.r, color.g, color.b)
        bgt:Show()
        if ( db.useBorders ) then
            _G["DHUD3_"..bar.."_Border"]:Show();
        else
            _G["DHUD3_"..bar.."_Border"]:Hide();
        end
    end
end

--[[
    formatTime: Format a time in seconds to Hours, minutes, seconds
    arguments:
        secs: time in seconds to format
    returns:
        Formated time
]]--
function DHUD3:formatTime(secs)
	if secs >= 86400 then
		return format('%dd', math.ceil(secs/86400))
	elseif secs >= 3600 then
		return format('%dh', math.ceil(secs/3600))
	elseif secs >= 60 then
		return format('%dm', math.ceil(secs/60))
	else
		return format("%d", secs)
	end
end

--[[
        OnInitialize: Function called when the addon is initialized
        --]]
function DHUD3:OnInitialize()
	--DHUD3:ToggleDebugLog(true)
    
    -- Get layout information
    local mainFrame = self.layout.mainFrame;
    local subFrames = self.layout.frames;
	-- Main Frame
    local point, parent, relative, x, y, width, height = unpack(mainFrame);
    ref = CreateFrame ("Frame", "DHUD3_mainFrame", _G[parent]);
    ref:ClearAllPoints();
    ref:SetPoint(point, parent, relative, x, y);
    ref:SetHeight(height);
    ref:SetWidth(width);
    ref:EnableMouse(false);
    --ref:Show();
    
    -- Left and right frames were the bars and texts are placed
    for k, v in pairs(subFrames) do
        point, parent, relative, x, y, width, height = unpack(v);
        ref = CreateFrame("Frame", "DHUD3_"..k, _G["DHUD3_"..parent]);
        ref:ClearAllPoints();
        ref:SetPoint(point, _G["DHUD3_"..parent], relative, x, y);
        ref:SetHeight(height);
        ref:SetWidth(width);
        ref:EnableMouse(false);
    end
	
    --Manage the Database
	self.db = LibStub("AceDB-3.0"):New("DHUD3DB", defaults, "Default");
	self.db.RegisterCallback( DHUD3, "OnProfileChanged", "Refresh" );
	self.db.RegisterCallback( DHUD3, "OnProfileCopied", "Refresh" );
	self.db.RegisterCallback( DHUD3, "OnProfileReset", "Refresh" );
    db = self.db.profile;
	
    self.optionsFrames = {};
	
    
    --Add the options table to the addon menu in blizz addon interface (thanks Nevcariel)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("DHUD3", getOptions);
    self.optionsFrames.DHUD3 = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("DHUD3", nil, nil, "layout");
    self:RegisterChatCommand("DHUD3", "ChatCommand");
    -- Add info to the Options array for the Profile Options
    self:RegisterModuleOptions("Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db), "Profiles");
    
    
    -- MyAddons Support
    myAddons();
end

-- Function to call when the chat command is entered (thanks Nevcariel)
function DHUD3:ChatCommand(input)
	----DHUD3:Debug("ChatCommand", input)
    -- open the profiles tab before, so the menu expands
    --InterfaceOptionsFrame_OpenToCategory(DHUD3.optionsFrames.Profiles);
    InterfaceOptionsFrame_OpenToCategory(DHUD3.optionsFrames.DHUD3);
end

-- Function to register other modules options with the core options table (thanks Nevcariel)
function DHUD3:RegisterModuleOptions(name, optionTbl, displayName)
	moduleOptions[name] = optionTbl;
	self.optionsFrames[name] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("DHUD3", displayName, "DHUD3", name);
end

function DHUD3:OnEnable() -- ok
    ----self:Debug("OnEnable")
	if ( not isSetup ) then
        DHUD3_mainFrame:Show();
        DHUD3_mainFrame:SetScale(db.scale);
        DHUD3:updateAlpha(0);
        isSetup = true;
    end  
end

--Reset the init variables so the addon will reload correctly when re-enabled
function DHUD3:OnDisable()
    ----self:Debug("OnDisable")
    DHUD3_mainFrame:Hide();
end

 function DHUD3:OnProfileChanged()
	DHUD3:Debug("OnProfileChanged");
	DHUD3:setup();
end

function DHUD3:GetModuleEnabled(module)
	return db.modules[module];
end

function DHUD3:SetModuleEnabled(module, value)
	local old = db.modules[module];
	db.modules[module] = value;
	if ( old ~= value ) then
		if value then
			self:EnableModule(module);
		else
			self:DisableModule(module);
		end
	end
end

function DHUD3:Refresh()
	db = self.db.profile;
	DHUD3_mainFrame:SetScale(db.scale);
	self:updateAlpha(0);
	for k,v in self:IterateModules() do
		if self:GetModuleEnabled(k) and not v:IsEnabled() then
			self:EnableModule(k);
		elseif not self:GetModuleEnabled(k) and v:IsEnabled() then
			self:DisableModule(k);
		end
		if type(v.Refresh) == "function" then
			v:Refresh();
		end
	end
	
end

function DHUD3:getScale()
    
    return db.scale;

end
